#include "StdAfx.h"
#include "CECMgr.h"

//Common error codes returned by MX_CECMgr_XXX functions
//0 - No Error
//1 - Initialization not called
//2 - Initialization failed
//3 - Invalid parameters
//4 - Buffer too small


CCECMgr::CCECMgr(void)
{
	m_hDLLInst = NULL;
}

CCECMgr::~CCECMgr(void)
{
}

BOOL CCECMgr::Init()
{
	BOOL bSuccess = FALSE;

	do
	{
		if(!m_hDLLInst)
			m_hDLLInst = ::LoadLibrary(CEC_DLL);

		if (!m_hDLLInst)
			break;

		if(m_hDLLInst)
		{
			m_pfnCECMgr_Init				= (MX_CECMGR_INIT)GetProcAddress(m_hDLLInst, "MX_CECMgr_Init");
			m_pfnCECMgr_Uninit				= (MX_CECMGR_UNINIT)GetProcAddress(m_hDLLInst, "MX_CECMgr_Uninit");
			m_pfnCECMgr_SetDiscoverDevice	= (MX_CECMGR_SETDISCOVERDEVICE)GetProcAddress(m_hDLLInst, "MX_CECMgr_SetDiscoverDevice");
			m_pfnCECMgr_HasReadData			= (MX_CECMGR_HASREADDATA)GetProcAddress(m_hDLLInst, "MX_CECMgr_HasReadData");
			m_pfnCECMgr_ReadData			= (MX_CECMGR_READDATA)GetProcAddress(m_hDLLInst, "MX_CECMgr_ReadData");
			m_pfnCECMgr_WriteData			= (MX_CECMGR_WRITEDATA)GetProcAddress(m_hDLLInst, "MX_CECMgr_WriteData");
		}
		
		if (!m_pfnCECMgr_Init				||
			!m_pfnCECMgr_Uninit				||
			!m_pfnCECMgr_SetDiscoverDevice	||
			!m_pfnCECMgr_HasReadData		||
			!m_pfnCECMgr_ReadData			||
			!m_pfnCECMgr_WriteData)
			break;

		if (m_pfnCECMgr_Init())
			break;

		bSuccess = TRUE;

	} while(FALSE);

	return bSuccess;
}

void CCECMgr::Uninit()
{
	if(!m_hDLLInst)
		return;

	m_pfnCECMgr_Uninit();
}

BOOL CCECMgr::SetDiscoverDevice(int iDevIndex)
{
	if(!m_hDLLInst)
		return FALSE;

	if (m_pfnCECMgr_SetDiscoverDevice(iDevIndex))
		return FALSE;

	return TRUE;
}

BOOL CCECMgr::HasReadData(BOOL *pbStatus)
{
	if(!m_hDLLInst)
		return FALSE;

	if (m_pfnCECMgr_HasReadData(pbStatus))
		return FALSE;

	return TRUE;
}

BOOL CCECMgr::ReadData(BYTE* pbData, int* piNumData)
{
	if(!m_hDLLInst)
		return FALSE;

	if (m_pfnCECMgr_ReadData(pbData, piNumData))
		return FALSE;

	return TRUE;
}

BOOL CCECMgr::WriteData(int iTargetDev, BYTE* pbData, int iNumData)
{
	if(!m_hDLLInst)
		return FALSE;

	if (m_pfnCECMgr_WriteData(iTargetDev, pbData, iNumData))
		return FALSE;

	return TRUE;
}

