
// MX_CECUtilDlg.h : header file
//

#pragma once
#include <initguid.h> //a must to be able to use other filter interface
#include ".\Thread\EventThread.h"
#include "afxwin.h"

#define START_EVENT_POLL					0x100

#define SAFE_RELEASE(x)			{ if (x) { x->Release(); x = NULL; } }
#define SAFE_DELETE(x)			{ if (x) { delete(x); x=NULL; } }
#define SAFE_ARRAYDELETE(x)		{ if (x) { delete[] x; x = NULL; } }
#define SAFE_ADDREF(x)			{ if (x) { x->AddRef(); } }

#define NUM_LOGICAL_DEV		    16
#define MAX_NUM_CEC_DATA		16
#define MAX_NUM_CMD				24
#define CEC_DATA_FILE			L"CECData.ini"
#define INI_SEC_MAIN			L"Main"
#define INI_SEC_CMD				L"Command%d"
#define INI_CMD_NAME			L"Name"
#define INI_CMD_BCAST			L"Broadcast"
#define INI_CMD_NUMDATA			L"NumData"
#define INI_CMD_VAL				L"Data%d"

#define CEC_MESSAGE_DATA_START	WM_USER + 100
#define CEC_MESSAGE_HAS_DATA	WM_USER + 101
#define CEC_MESSAGE_DATA_END	WM_USER + 102

#define TIMER_DISCOVER_DEVICE		1000
#define TIMER_DISCOVER_DURATION		1000

typedef enum
{
	LOGIC_DEV_0_TV,
	LOGIC_DEV_1_DVR1,
	LOGIC_DEV_2_DVR2,
	LOGIC_DEV_3_TUNER1,
	LOGIC_DEV_4_PLYR1,
	LOGIC_DEV_5_AUDIO,
	LOGIC_DEV_6_TUNER2,	
	LOGIC_DEV_7_TUNER3,	
	LOGIC_DEV_8_PLYR2,
	LOGIC_DEV_9_DVR3,
	LOGIC_DEV_A_TUNER4,
	LOGIC_DEV_B_PLYR3,
	LOGIC_DEV_C_RSVP,
	LOGIC_DEV_D_RSVP,
	LOGIC_DEV_E_FREE,
	LOGIC_DEV_F_BCAST
} LOGIC_DEV;

typedef struct 
{
	int		iNumData;
	BYTE	iData[MAX_NUM_CEC_DATA];
	wchar_t wszCaptionName[80];
	BOOL	bBroadcast;
} INDV_CEC_DATA;

typedef struct 
{
	INDV_CEC_DATA	cmd[MAX_NUM_CMD];
} CEC_DATA;

class CPollEventMgr;

// CMX_CECUtilDlg dialog
class CMX_CECUtilDlg : public CDialog
{
// Construction
public:
	CMX_CECUtilDlg(CWnd* pParent = NULL);	// standard constructor
	void ReadCECData();
	void SaveCECData();
	void InsertLog(CString szMsg);
	void RefreshControl();
	void GetLogicalDevName(int iDevAddr, CString *pStrName);
	void SendCECData(int iWhichCmd);
	void HasCECData(BYTE iData);
	int  GetDiscoverDevPrgs() { return m_iCurrDiscoverDevCounter; }
	void ResetDiscoveredDevice();

// Dialog Data
	enum { IDD = IDD_MX_CECUTIL_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnBnClickedButtonDiscoverDev();
	afx_msg void OnBnClickedButtonCmd1();
	afx_msg void OnBnClickedButtonCmd2();
	afx_msg void OnBnClickedButtonCmd3();
	afx_msg void OnBnClickedButtonCmd4();
	afx_msg void OnBnClickedButtonCmd5();
	afx_msg void OnBnClickedButtonCmd6();
	afx_msg void OnBnClickedButtonCmd7();
	afx_msg void OnBnClickedButtonCmd8();
	afx_msg void OnBnClickedButtonCmd9();
	afx_msg void OnBnClickedButtonCmd10();
	afx_msg void OnBnClickedButtonCmd11();
	afx_msg void OnBnClickedButtonCmd12();
	afx_msg void OnBnClickedButtonCmd13();
	afx_msg void OnBnClickedButtonCmd14();
	afx_msg void OnBnClickedButtonCmd15();
	afx_msg void OnBnClickedButtonCmd16();
	afx_msg void OnBnClickedButtonCmd17();
	afx_msg void OnBnClickedButtonCmd18();
	afx_msg void OnBnClickedButtonCmd19();
	afx_msg void OnBnClickedButtonCmd20();
	afx_msg void OnBnClickedButtonCmd21();
	afx_msg void OnBnClickedButtonCmd22();
	afx_msg void OnBnClickedButtonCmd23();
	afx_msg void OnBnClickedButtonCmd24();
	afx_msg void OnBnClickedButtonCmdEdit();

	afx_msg LRESULT OnCECMessage(UINT id);
	afx_msg LRESULT OnCECStartMessage(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnCECDataMessage(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnCECEndMessage(WPARAM wParam, LPARAM lParam);

private:
	BOOL				m_bInitialized;
	BOOL				m_bDiscoverDevice;
	CPollEventMgr*		m_pPollMgr;
	CEC_DATA			m_CECData;
	BYTE				m_iRcvCECData[MAX_NUM_CEC_DATA];
	BYTE				m_iDiscoveredDevLogicalAddr[NUM_LOGICAL_DEV];
	BYTE				m_iNumRcvCECData;
	BYTE				m_iCurrSelDevIndex;
	BYTE				m_iCurrDiscoverDevCounter;
	BYTE				m_iNumDiscoveredDevLogicalAddr;

public:
	CListBox m_LogListBoxCtrl;
	afx_msg void OnTimer(UINT_PTR nIDEvent);
	CComboBox m_DeviceCmbCtrl;
	afx_msg void OnCbnSelchangeComboCurrDev();
	afx_msg void OnBnClickedButtonLogClear();
};

class CPollEventMgr : public CEventThread
{
public:
	CPollEventMgr(CWnd* pParent);
	void ResetPollCounter();
	virtual INT_32 ServiceEvent(CMsgEvent *pEvent);

private:
	CWnd*				m_pPollParent;
	int					m_iPollInterval;
};
