// EventThread.cpp: implementation of the CEventThread class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EventThread.h"

#define SAFEDELETE(X) if(X){delete(X); X=0;}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEventThread::CEventThread()
{
	m_pEventThread = NULL;
	m_pEventThreadSemaphore = NULL;
	m_pEventThreadQueue = NULL;
	m_pEventThreadEnd = NULL;

	m_pEventThreadSemaphore	= new Semaphore();
	m_pEventThreadQueue		= new Queue < CMsgEvent * >();
	m_pEventThreadEnd = new Semaphore();
}

CEventThread::~CEventThread()
{
	if(m_pEventThread)
	{
		StopEventThread();
		m_pEventThread->Join();
		SAFEDELETE(m_pEventThread);
	}

	SAFEDELETE(m_pEventThreadSemaphore);
	SAFEDELETE(m_pEventThreadQueue);
	SAFEDELETE(m_pEventThreadEnd);
}

void CEventThread::StartEventThread()
{
	if(!m_pEventThread)
	{
		m_pEventThread = Thread::CreateThread();
		m_pEventThread->Create(CEventThread::EventThreadFunc,this);
	}
}

void CEventThread::StopEventThread()
{
	if(m_pEventThread)
	{
		m_pEventThreadEnd->Signal();
	}
}

void CEventThread::EventThreadFunc(void* pArg)
{
	CEventThread*	pThis = (CEventThread*) pArg;
	CMsgEvent*		pMsgEvent;
	INT_32			rtnVal = 0x0;
	int iWaitResult;

	HANDLE		WaitObjects[] = { pThis->m_pEventThreadEnd->m_sem, pThis->m_pEventThreadSemaphore->m_sem };

	pThis->InitEventThread();
	while(rtnVal == 0)
	{  
		// serviceEvent will return 1 if error or timeout
		if(pThis)
		{ 
			// the windows message pump
			MSG msg; 
			if( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );
			}

			iWaitResult = WaitForMultipleObjects(2, WaitObjects, FALSE, 10);
			if(iWaitResult == WAIT_OBJECT_0 + 1)
			{
				/// 02-10-03 (dj) Add, Safer.
				if( !pThis->m_pEventThreadQueue )
					break;
				/// 02-10-03 (dj)

				pMsgEvent = pThis->m_pEventThreadQueue->Read();
				if (pMsgEvent)
				{
					rtnVal = pThis->ServiceEvent(pMsgEvent);
				}
			} else if(iWaitResult == WAIT_OBJECT_0)
			{
				// End Thread;
				break;
			}
		}
	}
	pThis->DeinitEventThread();
}

INT_32 CEventThread::AcceptEvent(CMsgEvent *pEvent)
{
	/// 02-10-03 (dj) Add, Safer.
	if( !pEvent )
		return 0;
	/// 02-10-03 (dj)

	if(m_pEventThreadQueue)
	{
		m_pEventThreadQueue->Write(pEvent);
   
		if(m_pEventThreadSemaphore)
			m_pEventThreadSemaphore->Signal();
	}
	else
		SAFEDELETE(pEvent);
   
	return 0;
}

INT_32 CEventThread::ServiceEvent(CMsgEvent *pEvent)
{
	if (!pEvent)
		return 255;

	/*
	switch (pEvent->Type())
	{
		case END_EVENT_THREAD:
			SAFEDELETE(pEvent);
			return -1;
		break;
	}
	*/

	SAFEDELETE(pEvent);

	return 0;
}

void CEventThread::InitEventThread()
{
}

void CEventThread::DeinitEventThread()
{
}
