#include "stdafx.h"
#include <process.h>

#include "win32thread.h"


win32Thread::win32Thread():Thread()
{
	m_priority		= Normal;
	m_threadHandle	= NULL;	
	m_threadId		= 0;
    m_function      = NULL;
    m_arg           = NULL;
}

win32Thread::~win32Thread()
{
    if(m_threadHandle)
    {
	    CloseHandle(m_threadHandle);
    }
}

unsigned long __stdcall win32Thread::internalThreadFunction(void* arg)
{
    win32Thread* thread = (win32Thread*) arg;

    return thread->InternalThreadFunction();
}

UINT_32 win32Thread::InternalThreadFunction()
{
    m_function(m_arg);

    return 0;
}

bool win32Thread::Create(thread_function function, void* arg)
{
	bool result = false;

    m_function      = function;
    m_arg           = arg;

	//m_threadHandle = (HANDLE) _beginthreadex(
	/*LEAK*/m_threadHandle = ::CreateThread(
									NULL,
									0,
									internalThreadFunction,
									this,
									0,
									&m_threadId);
	if(m_threadHandle)
	{
		result = true;
	}
	
	return result;
}

void win32Thread::Destroy()
{
	 TerminateThread(m_threadHandle, 0);
}

void win32Thread::Suspend()
{
	 SuspendThread(m_threadHandle);
}

void win32Thread::Resume()
{
	ResumeThread(m_threadHandle);
}

void win32Thread::Join()
{
	// We need to do a MsgWaitForMultipleObjects instead of WaitForObjects, as
	// we cannot be certain that the thread does not create windows, or be involved
	// in windows messaging loops. Otherwise, deadlock would occur
	while(TRUE)
	{
		MSG msg;
		DWORD result;
		// Read all of the messages in this next loop, 
        // removing each message as we read it.
        while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) 
            DispatchMessage(&msg); 

		// Wait for any message sent or posted to this queue
        // or the thread handle to be signaled		
		result = MsgWaitForMultipleObjects(1, &m_threadHandle, FALSE, INFINITE, QS_ALLINPUT);
        // The result tells us the type of event we have.
        if (result == (WAIT_OBJECT_0 + 1))
        {
            // New messages have arrived. 
            // Continue to the top of the always while loop to 
            // dispatch them and resume waiting.
            continue;
        } 
        else 
        { 
			break;		// thread handle is available for us
        } // End of else clause.
    } // End of the always while loop. 
}

Priority win32Thread::GetPriority() const
{
	return (Priority) GetThreadPriority(m_threadHandle);
}

Priority win32Thread::SetPriority(Priority priority)
{
	Priority old = (Priority) GetThreadPriority(m_threadHandle);

	SetThreadPriority(m_threadHandle, priority);

	return old;
}

